"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeHandler = exports.Iteration = void 0;
const handler_1 = require("./handler");
const handler_action_enum_1 = require("./handler-action.enum");
const map_1 = require("./handler/map");
const pass_through_1 = require("./handler/pass-through");
const recover_1 = require("./handler/recover");
const side_effect_1 = require("./handler/side-effect");
const tap_1 = require("./handler/tap");
var Iteration;
(function (Iteration) {
    Iteration["THROW"] = "throw";
    Iteration["CONTINUE"] = "continue";
    Iteration["BREAK"] = "break";
    Iteration["RETURN"] = "return";
})(Iteration = exports.Iteration || (exports.Iteration = {}));
async function executeHandler(handler, context, debug) {
    const nextError = context.error;
    if (handler.scope && !(context.error instanceof handler.scope)) {
        debug('skipping handler because its scope did not match');
        return { iteration: Iteration.CONTINUE, error: nextError };
    }
    try {
        if (handler.predicate &&
            !(await handler.predicate.call(context.self, context.error, ...context.parameters))) {
            debug('skipping handler because its predicate return false');
            return { iteration: Iteration.CONTINUE, error: nextError };
        }
    }
    catch (error) {
        return {
            iteration: Iteration.THROW,
            error: new handler_1.PredicateError({ trigger: context.error, cause: error, handler }),
        };
    }
    switch (handler.action) {
        case handler_action_enum_1.HandlerAction.MAP:
            return (0, map_1.mapHandler)(debug, context, handler);
        case handler_action_enum_1.HandlerAction.RECOVER:
            return (0, recover_1.recoverHandler)(debug, context, handler);
        case handler_action_enum_1.HandlerAction.SIDE_EFFECT:
            return (0, side_effect_1.sideEffectHandler)(debug, context, handler);
        case handler_action_enum_1.HandlerAction.TAP:
            return (0, tap_1.tapHandler)(debug, context, handler);
        case handler_action_enum_1.HandlerAction.PASS_THROUGH:
            return (0, pass_through_1.passThroughHandler)(debug, context, handler);
        default:
            throw new RangeError(`Unexpected handler action: ${handler.action}`);
    }
}
exports.executeHandler = executeHandler;
