"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandleError = void 0;
const debug_1 = require("./debug");
const run_error_handlers_1 = require("./run-error-handlers");
function HandleError(...handlers) {
    return function mapError(target, propertyKey, descriptor) {
        const wrappedFunction = descriptor.value;
        async function wrapper(...parameters) {
            const dbg = debug_1.debug.extend(`${this.constructor.name}.${wrappedFunction.name}`);
            try {
                dbg('executing function');
                const result = (await wrappedFunction.apply(this, parameters));
                dbg('executed function without errors');
                return result;
            }
            catch (error) {
                dbg('function threw %s', error.name);
                const context = { self: this, parameters, error: error };
                return await (0, run_error_handlers_1.runErrorHandlers)(context, handlers, dbg);
            }
        }
        descriptor.value = wrapper;
    };
}
exports.HandleError = HandleError;
