"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleCallbackError = void 0;
const execute_handler_1 = require("../execute-handler");
const callback_error_action_enum_1 = require("./callback-error-action.enum");
function handleCallbackError({ trigger, callbackError, WrapperClass, handler, }, debug) {
    debug('%s handler threw an error: %s', handler.action, callbackError.message);
    const callbackErrorAction = 'onCallbackError' in handler ? handler.onCallbackError : callback_error_action_enum_1.CallbackErrorAction.THROW_WRAPPED;
    switch (callbackErrorAction) {
        case callback_error_action_enum_1.CallbackErrorAction.THROW_WRAPPED: {
            debug('error action is set to %s, throwing a %s instead', callbackErrorAction, WrapperClass.name);
            return {
                iteration: execute_handler_1.Iteration.THROW,
                error: new WrapperClass({
                    trigger,
                    cause: callbackError,
                    handler,
                }),
            };
        }
        case callback_error_action_enum_1.CallbackErrorAction.THROW: {
            debug('error action is set to %s, throwing the error from the callback', callbackErrorAction);
            return {
                iteration: execute_handler_1.Iteration.THROW,
                error: callbackError,
            };
        }
        case callback_error_action_enum_1.CallbackErrorAction.THROW_TRIGGER: {
            debug('error action is set to %s, throwing the trigger error', callbackErrorAction, WrapperClass.name);
            return {
                iteration: execute_handler_1.Iteration.THROW,
                error: trigger,
            };
        }
        default:
            throw new RangeError(`Unexpected action for callback error handling: ${callbackErrorAction}`);
    }
}
exports.handleCallbackError = handleCallbackError;
