"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapHandler = void 0;
const execute_handler_1 = require("../../execute-handler");
const handle_callback_error_1 = require("../handle-callback-error");
const map_error_1 = require("./map.error");
async function mapHandler(debug, context, handler) {
    try {
        const mappedError = await handler.callback.call(context.self, context.error, ...context.parameters);
        debug('error successfully mapped from %s to %s', context.error.name, mappedError.name);
        if (handler.continue) {
            debug(`'continue' is set, proceeding with remaining handlers`);
            return { iteration: execute_handler_1.Iteration.CONTINUE, error: mappedError };
        }
        else {
            debug(`'continue' is not set, skipping remaining handlers`);
            return { iteration: execute_handler_1.Iteration.BREAK, error: mappedError };
        }
    }
    catch (callbackError) {
        return (0, handle_callback_error_1.handleCallbackError)({
            trigger: context.error,
            callbackError: callbackError,
            WrapperClass: map_error_1.MapError,
            handler,
        }, debug);
    }
}
exports.mapHandler = mapHandler;
