"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runErrorHandlers = void 0;
const execute_handler_1 = require("./execute-handler");
async function runErrorHandlers(context, handlers, parentDebugger) {
    let error = context.error;
    parentDebugger('executing %d handler(s)', handlers.length);
    Loop: for (const [index, handler] of handlers.entries()) {
        const debug = parentDebugger.extend(`handler:${index}`);
        debug(`executing '%s' handler`, handler.action);
        const result = await (0, execute_handler_1.executeHandler)(handler, Object.assign(Object.assign({}, context), { error }), debug);
        switch (result.iteration) {
            case execute_handler_1.Iteration.CONTINUE:
                error = result.error;
                continue;
            case execute_handler_1.Iteration.BREAK:
                error = result.error;
                break Loop;
            case execute_handler_1.Iteration.RETURN:
                return result.value;
            case execute_handler_1.Iteration.THROW:
                throw result.error;
            default:
                throw new RangeError(`Unexpected iteration result type: ${result.iteration}`);
        }
    }
    throw error;
}
exports.runErrorHandlers = runErrorHandlers;
